package com.ruoyi.analysis.config;

import com.alibaba.fastjson.JSON;
import com.ruoyi.analysis.constants.*;
import com.ruoyi.analysis.domain.VehicleMessage;
import com.ruoyi.analysis.handler.MessageHandler;
import com.ruoyi.analysis.handler.MessageVerify;
import com.ruoyi.analysis.hbase.service.HBaseService;
import com.ruoyi.common.redis.service.RedisService;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.annotation.TopicPartition;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Optional;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * Kafka消费者报文
 **/
@Slf4j
@Component
public class KafkaConsumer {
    /**
     * 注入线程池
     */
    @Autowired
    private ThreadPoolExecutor threadPoolExecutor;

    /**
     * 注入redis
     */
    @Autowired
    private RedisService redisService;
    /**
     * 注入rabbitMQ
     */
    @Autowired
    private RabbitTemplate rabbitTemplate;
    /**
     * Hbase工具类
     */
    @Autowired
    private HBaseService hBaseService;
    /**
     * 初始化线程池
     */
    private ExecutorService executorService= Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors()*2);


    private SimpleDateFormat tableNameFmt = new SimpleDateFormat("yyyy-MM-dd");

    /**
     * kafka消息接收端
     */
    @KafkaListener(topicPartitions = {@TopicPartition(topic = "test",partitions = {"2"})})
    public void OnMessage(ConsumerRecord<?,?> consumerRecord, Acknowledgment acknowledgment) throws InterruptedException{
        try {
            //提交到线程池执行
            threadPoolExecutor.submit(()->{
                messageHandler(consumerRecord); //处理消息方法体;
            });
        }finally {
            acknowledgment.acknowledge();
        }

    }
    /**
     * 处理消息方法体
     */
    public void messageHandler(ConsumerRecord<?,?> consumerRecord){
        //调用Optional防止空指针异常
        Optional<?> value=Optional.ofNullable(consumerRecord.value());
        String decimalInfo=null;
        String StringMessage=null;
        if (value.isPresent()){
            //获取到消息的内容
            Object object = value.get();
            StringMessage = (String) object;


            log.info("报文内容:"+StringMessage);
            String content = StringMessage.substring(4, StringMessage.length() - 7);
            decimalInfo = MessageVerify.theDecimal(content);
            String vin = "";
            if (decimalInfo.contains(VehicleConstant.NETTY_WILL_CLOSE)) {
                vin = decimalInfo.substring(VehicleConstant.NETTY_WILL_CLOSE.length(), VehicleConstant.NETTY_WILL_CLOSE.length() + 17);
            }
            //车辆启动
            if (decimalInfo.contains(VehicleConstant.VEHICLE_START_SUF)) {

                vin = decimalInfo.substring(VehicleConstant.VEHICLE_START_SUF.length(), VehicleConstant.VEHICLE_START_SUF.length() + 17);

                log.info("车辆启动信息;vin:{};内容:{}", vin, decimalInfo);
                VehicleStatusEntity statusEntity = new VehicleStatusEntity(vin, VehicleConstants.VEHICLE_START, System.currentTimeMillis());
                rabbitTemplate.convertSendAndReceive(VehicleConstants.VEHICLE_STATUS_UPDATE_QUEUE, JSON.toJSONString(statusEntity),message -> {
                    message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
                    return message;
                });
                return;
            }
            //车辆停止
            if (decimalInfo.contains(VehicleConstant.VEHICLE_STOP_SUF)) {
                vin = decimalInfo.substring(VehicleConstant.VEHICLE_STOP_SUF.length(), VehicleConstant.VEHICLE_STOP_SUF.length() + 17);
                log.info("车辆停止信息;vin:{};内容:{}", vin, decimalInfo);
                VehicleStatusEntity statusEntity = new VehicleStatusEntity(vin, VehicleConstants.VEHICLE_END, System.currentTimeMillis());
                rabbitTemplate.convertSendAndReceive(VehicleConstants.VEHICLE_STATUS_UPDATE_QUEUE, JSON.toJSONString(statusEntity),message -> {
                    message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
                    return message;
                });
                return;
            }
            //车辆消息
            if (decimalInfo.contains(VehicleConstant.VEHICLE_MSG_SUF)) {
                vin = decimalInfo.substring(VehicleConstant.VEHICLE_MSG_SUF.length(), VehicleConstant.VEHICLE_MSG_SUF.length() + 17);
                //拼接上时间戳
                decimalInfo += System.currentTimeMillis();
                log.info("车辆普通消息;vin:{};内容:{}", vin, decimalInfo);
            }
        }

            log.info("topic:{},分区:{},消息内容:{}","vehicle_message:"+consumerRecord.partition(),decimalInfo);
            //将车辆消息转为实体类
            VehicleMessage vehicleMessage= MessageHandler.messageTranslated(decimalInfo);
            log.info("切割过后的报文:"+vehicleMessage);
            //车辆实时数据存入Redis
            redisService.setCacheObject(RedisConstant.CURRENT_INFO+":"+vehicleMessage.getVinCode(),JSON.toJSONString(vehicleMessage));
            if(vehicleMessage.getBatteryCurrent()!=null){
                //车辆故障处理
                rabbitTemplate.convertSendAndReceive(RabbitConstant.FAULT_HANDLER_QUEUE,JSON.toJSONString(decimalInfo),message -> {
                    message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
                    return message;
                });
            }
            //车辆围栏处理
            rabbitTemplate.convertSendAndReceive(RabbitConstant.FENCE_HANDLER_QUEUE,JSON.toJSONString(vehicleMessage),message -> {
                message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
                return message;
            });
            try {
                String tableName = HBaseConstant.HBASE_TABLE_PREFIX + tableNameFmt.format(new Date(vehicleMessage.getTime()));
                log.info("信息入库;database:{},vin:{},time:{}",tableName,vehicleMessage.getVinCode(),System.currentTimeMillis());
                hBaseService.insertRecordsForObj(tableName,vehicleMessage.getVinCode()+":"+vehicleMessage.getTime(),"analysis",vehicleMessage);
            } catch (IOException e) {
                log.error("添加到HBase出错:{}",e.getMessage());
            }
    }
}

